<?php

 /**
 * Core class for FramePress.
 *
 * This class is responsable of create all core class components and implement some genric functions
 * It also implements the flow for activation and deactivation of the plugin
 *
 * Licensed under The GPL v2 License
 * Redistributions of files must retain the above copyright notice.
 *
 * @link			none yet
 * @package			core
 * @subpackage		core.FramePress
 * @license			GPL v2 License

 * IMPORTANT NOTE: class name will be rewrited as core_[prefix] (see init.php file), to get unique class names between plugins.
 */

class PressbackupCore
{
	/**
	 * store the instance of this clase
	 * to use on core Classes
	 *
	 * @var Object
	 * @access public
	*/
	public static $instance = null;

	/**
	 * Instance of Path Class
	 *
	 * @var Object
	 * @access public
	*/
	public $Path = null;

	/**
	 * Instance of View Class
	 *
	 * @var Object
	 * @access public
	*/
	public $View = null;

	/**
	 * Instance of Page Class
	 *
	 * @var Object
	 * @access public
	*/
	public $Page = null;

	/**
	 * Instance of Action Class
	 *
	 * @var Object
	 * @access public
	*/
	public $Action = null;

	/**
	 * Instance of Config Class
	 *
	 * @var Object
	 * @access public
	*/
	public $Config = null;

	/**
	 * Instance of Html Class
	 *
	 * @var Object
	 * @access public
	*/
	public $Html = null;

	/**
	 * Name of main file, by default main.php
	 *
	 * @var String
	 * @access public
	*/
	public $main_file = null;


	/**
	 * Constructor. It wil create all the objects necesaries for FramePress
	 *
	 * @param string $main_file Name of the main file
	 * @access public
	*/
	public function __construct( $main_file = null )
	{
		global $FP_CONFIG;

		self::$instance = $this;

		$this->main_file = $main_file;

		//create an intance of each core lib
		
		require_once dirname($main_file) . DS . '.core' . DS . 'config.php';
		$this->Config = new PressbackupConfig();

		require_once dirname($main_file) . DS . '.core' . DS . 'path.php';
		$this->Path = new PressbackupPath( $this , $main_file );

		require_once dirname($main_file) . DS . '.core' . DS . 'action.php';
		$this->Action = new PressbackupAction( $this );

		require_once dirname($main_file) . DS . '.core' . DS . 'page.php';
		$this->Page = new PressbackupPage( $this );

		require_once dirname($main_file) . DS . '.core' . DS . 'view.php';
		$this->View = new PressbackupView( $this );

		require_once dirname($main_file) . DS . '.core' . DS . 'html.php';
		$this->Html = new PressbackupHtml( $this );

		//register activation and deactivation functions
		register_activation_hook(basename(dirname($main_file)) . DS . basename($main_file), array($this,'activation'));
		register_deactivation_hook(basename(dirname($main_file)) . DS . basename($main_file), array($this, 'deactivation'));

		//capture output
		add_action('admin_init', array($this, 'capture_output'));

		//load languages
		add_action('init', array($this, 'load_languages'));
	}

	/**
	 * Start the output capture to can use headers on the plugin
	 *
	 * @return void
	 * @access public
	*/
	public function capture_output ()
	{
		ob_start();
	}

	/**
	 * Start the output capture to can use headers on the plugin
	 *
	 * @return void
	 * @access public
	*/
	public function load_languages ()
	{
		if( $this->Config->read('use.i18n') ) {
			load_plugin_textdomain(strtolower($this->Config->read('prefix')), false, $this->Path->Dir['LANG'] );
		}
	}

	/**
	 * Call activation function
	 *
	 * @return void
	 * @access public
	*/
	public function activation ()
	{
		require_once ($this->Path->Dir['CONFIG'] . DS . 'activation.php');
		pb_on_activation();
	}

	/**
	 * Call deactivation function
	 *
	 * @return void
	 * @access public
	*/
	public function deactivation ()
	{
		$this->import('session.php');
		$Session = $this->Config->read('prefix') . 'Session';
		$this->Session = new $Session( $this );
		delete_option($this->Session->session_name);

		require_once ($this->Path->Dir['CONFIG'] . DS . 'activation.php');
		pb_on_deactivation();
	}

	/**
	 * Perform a import of a file on lib folder
	 *
	 * @param string $name the place for redirect
	 * @return void
	 * @access public
	*/
	public function import ($name, $type = 'Lib')
	{
		$file =  $this->Path->Dir['LIB'] . DS . $name;
		$default_file =  $this->Path->Dir['D_LIB'] . DS . $name;

		if(file_exists ($file) ) {
			require_once($file);
		}

		elseif(file_exists ($default_file)){
			require_once($default_file);
		}
	}

	/**
	 * Perform a redirect using headers
	 *
	 * @param array $url The place for redirect
	 * @param boolean $exitCondition to perform an exit after redirect
	 * @return void
	 * @access public
	*/
	public function redirect ($url=array(), $exit= true)
	{
		ob_end_clean();

		$href = html_entity_decode($this->Path->router($url));

		if($exit) {
			header('Location: '.$href);
			exit;
		} else {
			$this->kick_user($href);
		}
	}

	/**
	 * Close the HTTP conection to give back the control, and keep working
	 *
	 * @param string $href the place for redirect
	 * @return void
	 * @access public
	*/
	public function kick_user($href = null)
	{
		// Close user conection and keep working
		@ob_end_clean();
		@ob_start();
		@ignore_user_abort(true);
		header("Status: 302");
		header('Location: '.$href, true, 302);
		header("Content-Length: 0", true);
		header("Connection: close", true);
		echo str_repeat("\r\n", 128); // for IE
		@ob_end_flush();
		@ob_flush();
		@flush();
	}
}


?>
