<?php

/**
 * Settings Controller for Pressbackup.
 *
 * This Class provide a interface to display and manage Plugin settings
 * Also Manage the configurattion wizard
 *
 * Licensed under The GPL v2 License
 * Redistributions of files must retain the above copyright notice.
 *
 * @link			http://pressbackup.com
 * @package			controlers
 * @subpackage		controlers.settings
 * @since			0.1
 * @license			GPL v2 License
 */

class PressbackupSettings extends PressbackupController
{
	public $layout = 'principal';

	public $Msg = null;

	public $Session = null;

	public $Misc = null;

	public $fp = null;

	public function __construct (&$FramePress)
	{
		$this->fp = $FramePress;

		$FramePress->import('Misc.php');
		$this->Misc = new MiscPBLib();

		$FramePress->import('session.php');
		$this->Session = new PressbackupSession($FramePress);

		$FramePress->import('msg.php');
		$this->Msg = new PressbackupMsg($FramePress);
	}

	/**
	 * Redirect the user to de correct page
	 * If the user has not configured the plugin
	 * this function redirect directly to the config init page
	 *
	 * Note: this function is automaticaly called
	 * by pressing on settings menu link
	 */
	public function index()
	{
		$settings = get_option('pressbackup.preferences');

		if ($settings['configured']) {
			$this->redirect(array('function' => 'configService'));
		} else {
			$this->redirect(array('function' => 'wizardInit'));
		}
	}

	//----------------------------------------------------------------------------------------

	/**
	 * Start The configuration wizard
	 * Check all that all nedded libs are intalled
	 * Also check for perms on tmp folders
	 */
	public function wizardInit()
	{
		$settings = get_option('pressbackup.preferences');

		$need_to_update_preferences = false;

		$error = array();

		$show_page = 'welcome.for.registered.users';

		// Check for CURL
		if (!extension_loaded('curl')) {
			$error['curl'] = __('PHP-CURL Extension missing: You probably don\'t have the php-curl extension installed.  You will need to contact your hosting provider and ask them to install it for you','pressbackup');
			$show_page = 'wizard.errors';
		}

		//Check if user is member of pressbackup
		$srv = $this->Misc->serviceObject('pressbackup', array(null, null));
		$res = $srv->auth();
		if(!$error && !$res) {
			$show_page = 'welcome.for.new.users';
		} elseif (!$error) {
			$settings['membership']=$res['package'];
			$settings['Service']['pressbackup']['credential'] = base64_encode($res['username'] . '|AllYouNeedIsLove|' . $res['authkey']);
			$need_to_update_preferences = true;
		}

		//check for Zip Creation
		$shellzip = $this->Misc->checkShell('zip');
		$phpzip = class_exists('ZipArchive');
		if (!$phpzip && !$shellzip) {
			$error['zip'] = __('PHP-ZIP Extension missing: You probably don\'t have the php-zip extension installed. You will need to contact your hosting provider and ask them to install it','pressbackup');
			$show_page = 'wizard.errors';
		}
		elseif (!$phpzip) {
			$settings['compatibility']['zip'] = 20;
			$need_to_update_preferences = true;
		}
		else {
			$settings['compatibility']['zip'] = 10;
			$need_to_update_preferences = true;
		}

		$blog_url = get_bloginfo( 'wpurl' );

		//Check for SSL
		$parsed_url = parse_url($blog_url);
		if($parsed_url['scheme'] == 'https'){
			$settings['compatibility']['background'] = 20;
			$need_to_update_preferences = true;
		}

		// Check CURL local
		$this->import('SimpleCurl.php');
		$curl = new SimpleCurlPBLib();
		$curl->call(array(
			'url' => $blog_url,
			'parse.response' => false
		));

		if(!in_array($curl->response['code'], array(200, 301))) {
			$error['curl'] = __('CURL Permission error: You probably don\'t have permission to run curl locally.  You will need to contact your hosting provider and ask them if "loopback HTTP requests are enabled."','pressbackup');
			$show_page = 'wizard.errors';
		}

		//tmp dir
		if (!file_exists($this->Path->Dir['PBKTMP']) && !mkdir($this->Path->Dir['PBKTMP'])) {
			$error['tmpdir'] = sprintf(__('We were unable to create the FileStore directory "%1$s" Please make sure the permissions are correct on this directory or its parent directory.','pressbackup'), $this->Path->Dir['PBKTMP']);
			$show_page = 'wizard.errors';
		} else {
			@chmod($this->Path->Dir['PBKTMP'], 0777);
		}

		//log dir
		if (!file_exists($this->Path->Dir['LOGTMP']) && !mkdir($this->Path->Dir['LOGTMP'])) {
			$error['logdir'] = sprintf(__('We were unable to create the FileStore directory "%1$s" Please make sure the permissions are correct on this directory or its parent directory.','pressbackup'), $this->Path->Dir['LOGTMP']);
			$show_page = 'wizard.errors';
		} else {
			@chmod($this->Path->Dir['LOGTMP'], 0777);
		}

		//validate use Cron
		$checkCron = get_option('pressbackup.wizard.cronTaskStatus');
		//ya corrio el cron y no paso nada entonces el ajax lo pone como fail
		if ($checkCron == "fail") {
			//informamos al usuario del error (pero como warning y lo dejamos seguir)
			$error = __('Could not create a Cron Job Backup.', 'pressbackup');
			$this->Msg->set('warning', $error);
			//actualizamos el valor a success para que no vuelva a hacer el chequeo
			update_option('pressbackup.wizard.cronTaskStatus', 'success');
			//para las futuras creaciones de process en bg, usamos tipo medium
			$settings['compatibility']['background'] = 20;
			$need_to_update_preferences = true;

		}
		//si no hay otros errores y todavia no se testeo el cron  checkear
		elseif ($show_page != 'wizard.errors' && $checkCron  != "success" ){
			$this->import('Scheduler.php');
			$sch = new SchedulerPBLib();
			$sch->add('+4 seconds', 'pressbackup.wizard.doCronTask');
			$this->set('check_cron', true);
		}
 
		//update preferences
		if($need_to_update_preferences){
			update_option('pressbackup.preferences', $settings);
		}

		//Alert the user about possible problems
		if (strpos($_SERVER['SERVER_SOFTWARE'], 'iis') !== false) {
			$error = __('Sorry your web Server (Windows IIS) is not compatible with Pressbackup. You are welcome to use Pressbackup, but we cannot provide any support or guarantees for your system. PressBackup is designed to run on Linux, BSD, and other Unix based systems','pressbackup');
			$this->Msg->set('warning', $error);
		}

		if (ini_get('safe_mode')) {
			$error = __('Safe Mode is enabled. - You will need to contact your hosting provider and ask them if it is absolutely necessary for you to have safe_mode enabled. Most hosts should be able to disable it for you, or give you a good reason why it can\'t be disabled','pressbackup');
			$this->Msg->set('warning', $error);
		}


		//values to show on user create  form
		$ud=get_userdata(get_current_user_id());
		$values = $default_values= array('User' =>array('username'=> $ud->data->user_login, 'password'=> '','password_confirm'=> '', 'first_name'=> '', 'last_name'=> ''));
		if($data = $this->Session->read('createData')){
			$values['User'] = array_merge($default_values['User'], $data['User']);
			$this->Session->delete('createData');
		}
		//set errors from user create
		if ($errors = $this->Session->read('errors')) {
			foreach($errors as $e){
				$this->Msg->set('error', __($e, 'pressbackup'));
			}
			$this->Session->delete('errors');
		}

		$this->set('data', $values);
		$this->set('error', $error);
		$this->set('show_page', $show_page);
	}

	/*
	 * CreateAccount via API
	 */
	public function createAccount ($type = null)
	{
		$srv = $this->Misc->serviceObject('pressbackup', array(null, null));

		$srv->createAccount($type);
		if($srv->response['code'] != '200'){
			$res=@json_decode($srv->response['body'], true);
			$this->Session->write('errors', $res['invalidFields']);
			$this->Session->write('createData', $_POST['data']);
		}
		$this->redirect(array('function' => 'wizardInit'));
	}

	/*
	 * CreateAccount via API
	 */
	public function registerSite ()
	{
		$srv = $this->Misc->serviceObject('pressbackup', array(null, null));
		$srv->registerSite();
		$this->redirect(array('function' => 'wizardInit'));
	}
	

	/*
	 * Page to select storage service
	 */
	public function wizardService ()
	{
		$error = $data = array();

		//set errors
		if ($error = $this->Session->read('error_msg')) {
			$this->Msg->set('error',$error);
			$data = $this->Session->read('data');
			$this->Session->delete('data');
			$this->Session->delete('error_msg');
		}

		$srv = $this->Misc->serviceObject('pressbackup', array(null, null));
		$this->set('data', $data);
		$this->set('servicesLimit', $srv->getLimit('services'));
	}

	/**
	 * Store the credentials if they are correct
	 * or save the error and return to form page
	 */
	public function wizardServiceSave()
	{
		//Exit if wrong parameters
		if (!isset($_POST['data']) || !$this->checkFieldsService($_POST['data'])) {
			$this->redirect(array('function' => 'wizardService'));
		}

		$settings = get_option('pressbackup.preferences');

		$settings['Service']['pressbackup']['enabled'] = false;
		$settings['Service']['s3']['enabled'] = false;
		$settings['Service']['local']['enabled'] = false;
		$settings['Service']['dropbox']['enabled'] = false;

		if ( in_array('pressbackup', $_POST['data']['service']) ) {

			$settings['Service']['pressbackup']['enabled'] = true;
		}

		if (in_array('s3', $_POST['data']['service'])) {

			$PS3 = $_POST['data']['s3'];
			$settings['Service']['s3']['bucket_name'] = $PS3['bucketname'];
			$settings['Service']['s3']['region'] = $PS3['region'];
			$settings['Service']['s3']['credential'] = base64_encode($PS3['accessKey'] . '|AllYouNeedIsLove|' . $PS3['secretKey']);
			$settings['Service']['s3']['enabled'] = true;
		}

		if (in_array('local', $_POST['data']['service'])) {

			$PLOCAL = $_POST['data']['local'];
			$settings['Service']['local']['path'] = $PLOCAL['path'];
			$settings['Service']['local']['enabled'] = true;
		}

		update_option('pressbackup.preferences', $settings);

		if (in_array('dropbox', $_POST['data']['service'])) {

			$srv = $this->Misc->serviceObject('dropbox', array(null, null));
			$srv->auth(array('menu_type'=>'menu', 'controller' => 'settings', 'function' => 'dropboxAuthorize'));

		} else {

			$this->wizardSetDefaultSettings();
		}
	}


	/**
	 * Store the settings if they are correct
	 * or save the error and return to form page
	 */
	private function wizardSetDefaultSettings()
	{
		$settings = get_option('pressbackup.preferences');

		$this->import('Scheduler.php');
		$sch = new SchedulerPBLib();

		$srv = $this->Misc->serviceObject('pressbackup');

		$curdate = strtotime($this->Misc->midnightUTC());
		$settings['backup']['db']= array('time' => $srv->getLimit('time'), 'last_date' => $curdate);
		$settings['backup']['plugins']= array('time' => 24, 'last_date' => $curdate);
		$settings['backup']['themes']= array('time' => 24, 'last_date' => $curdate);
		$settings['backup']['uploads']= array('time' => 24, 'last_date' => $curdate);
		$settings['backup']['copies'] = $srv->getLimit('copies');

		$settings['configured'] = 1;

		update_option('pressbackup.preferences', $settings);


		//create a first new
		$sch->add('+1 seconds');

		//ignore settings and make a first backup with all
		$this->Session->write('backup.type', '7,3,5,1', true);

		//tell dash to load ajax to chk backup status
		$this->Session->write('dash.reload', 'dashboard');

		$srv->pingConfigured();

		$this->redirect(array('controller'=>'main', 'function'=>'dashboard'));
	}

	//----------------------------------------------------------------------------------------

	public function wizardStartCronTask ()
	{
		update_option('pressbackup.wizard.cronTaskStatus', 'success');
	}

	public function wizardChkCronTaskStatus ()
	{
		$status = get_option('pressbackup.wizard.cronTaskStatus');

		$response = array('status'=> 'fail');
		if ($status == 'success') {
			$response = array('status'=> 'ok');
		}

		//clean output buffer
		ob_end_clean();

		//send json
		header("Status: 200", true);
		header("Content-type: application/json", true);
		echo json_encode($response);
		exit;
	}

	//when wizardChkCronTaskStatus ajax response is X times 'fail'
	//this function must be called to set the status of the process
	//to fail
	public function wizardSetCronTaskStatusFail ()
	{
		update_option('pressbackup.wizard.cronTaskStatus', 'fail');
		$this->redirect(array('controller'=>'settings', 'function'=>'wizardInit'));
	}

	//----------------------------------------------------------------------------------------

	/**
	 * Show the form to modify the service:
	 * S3, Pressbackup Pro, localhost, Dropbox, etc
	 */
	public function configService()
	{
		//set errors and data
		if ($error = $this->Session->read('error_msg')) {
			$this->Msg->set('error',$error);
			$data = $this->Session->read('data');
			$this->Session->delete('data');
			$this->Session->delete('error_msg');
		}
		//if no error, display settings info
		else {
			$settings = get_option('pressbackup.preferences');
			$enabled_services = $this->Misc->currentService();
			
			$dataService = $dataS3 = $dataLocal = array();
			foreach($enabled_services as $service){
				$dataService['service'][] = $service['id'];
				if ($service['id'] == 's3') {
					$credentials = explode('|AllYouNeedIsLove|', base64_decode( $service['credentials']));
					$dataS3['s3']['accessKey'] = $credentials[0];
					$dataS3['s3']['secretKey'] = $credentials[1];
					$dataS3['s3']['region'] = $service['region'];
				}
				if ($service['id'] == 'local') {
					$dataLocal['local']['path'] = $service['path'];
				}
			}
			$data = array_merge($dataService, $dataS3, $dataLocal);
		}

		$srv = $this->Misc->serviceObject('pressbackup');
		$this->set('data', $data);
		$this->set('servicesLimit', $srv->getLimit('services'));
	}

	/**
	 * Store the modified credentials if they are correct
	 * or save the error and return to form page
	 */
	public function configServiceSave()
	{
		//Exit if wrong parameters
		if (!isset($_POST['data']) || !$this->checkFieldsService($_POST['data'])) {
			$this->Session->write('data', $_POST['data']);
			$this->redirect(array('function' => 'configService'));
		}

		$settings = get_option('pressbackup.preferences');

		$settings['Service']['pressbackup']['enabled'] = false;
		$settings['Service']['s3']['enabled'] = false;
		$settings['Service']['local']['enabled'] = false;
		$settings['Service']['dropbox']['enabled'] = false;

		if ( in_array('pressbackup', $_POST['data']['service']) ) {

			$settings['Service']['pressbackup']['enabled'] = true;
		}

		if (in_array('s3', $_POST['data']['service'])) {

			$PS3 = $_POST['data']['s3'];
			$settings['Service']['s3']['bucket_name'] = $PS3['bucketname'];
			$settings['Service']['s3']['region'] = $PS3['region'];
			$settings['Service']['s3']['credential'] = base64_encode($PS3['accessKey'] . '|AllYouNeedIsLove|' . $PS3['secretKey']);
			$settings['Service']['s3']['enabled'] = true;
		}

		if (in_array('local', $_POST['data']['service'])) {

			$PLOCAL = $_POST['data']['local'];
			$settings['Service']['local']['path'] = $PLOCAL['path'];
			$settings['Service']['local']['enabled'] = true;
		}

		update_option('pressbackup.preferences', $settings);

		//add schedule backup job, for users from versions <=1.5.1
		$this->import('Scheduler.php');
		$sch = new SchedulerPBLib();
		$sch->add();

		$this->Session->delete('dash.service');

		if (in_array('dropbox', $_POST['data']['service'])) {
			$srv = $this->Misc->serviceObject('dropbox', array(null, null));
			$srv->auth(array('menu_type'=>'menu', 'controller' => 'settings', 'function' => 'dropboxAuthorize'));
		} else {
			$this->Session->write('general_msg', __('Service Saved!','pressbackup'));
			$this->redirect(array('controller'=> 'main', 'function' => 'dashboard'));
		}
	}

	/**
	 * Show the form to modify prefered settings
	 * as backup type, schedule time, etc
	 */
	public function configSettings()
	{
		if ($error = $this->Session->read('error_msg')) {
			$this->Msg->set('error',$error);
			$this->Session->delete('error_msg');
		}

		$srv = $this->Misc->serviceObject('pressbackup');

		$this->set('settings', get_option('pressbackup.preferences'));
		$this->set('time', $srv->getLimit('time'));
	}

	/**
	 * Store the modified settings if they are correct
	 * or save the error and return to form page
	 */
	public function configSettingsSave()
	{
		//exit if no data or invalid
		if (!isset($_POST['data']) || !$this->checkFieldsSettings($_POST['data'])) {
			$this->redirect(array('function' => 'configSettings'));
		}
		$data = $_POST['data']['backup'];

		$settings = get_option('pressbackup.preferences');
		$srv = $this->Misc->serviceObject('pressbackup');

		$time = $srv->getLimit('time');
		//time
		$curdate = strtotime($this->Misc->midnightUTC());
		$settings['backup']['db']['time'] = max($data['db']['time'], $time);
		$settings['backup']['plugins']['time'] = max($data['plugins']['time'], $time);
		$settings['backup']['themes']['time'] = max($data['themes']['time'], $time);
		$settings['backup']['uploads']['time'] = max($data['uploads']['time'], $time);

		update_option('pressbackup.preferences', $settings);

		$this->Session->write('general_msg', __('Preferences Saved!','pressbackup'));
		$this->redirect(array('controller'=> 'main', 'function' => 'dashboard'));
	}


	/**
	 * Show the form to modify prefered conpatibility settings
	 */
	public function configCompatibility()
	{
		$settings = get_option('pressbackup.preferences');

		$binaries = false;
		if ($this->Misc->checkShell('php')) {
			$binaries = true;
		}

		$this->set('binaries', $binaries);
		$this->set('settings', $settings);
	}

	/**
	 * Store the modified settings if they are correct
	 * or save the error and return to form page
	 */
	public function configCompatibilitySave()
	{
		$settings = get_option('pressbackup.preferences');

		if (isset($_POST['data']['compatibility']['background'])) {
			$settings['compatibility']['background'] = $_POST['data']['compatibility']['background'];
		}

		if (isset($_POST['data']['compatibility']['zip'])) {
			$settings['compatibility']['zip'] = $_POST['data']['compatibility']['zip'];
		}

		update_option('pressbackup.preferences', $settings);

		$this->Session->write('general_msg', __('Preferences Saved!','pressbackup'));
		$this->redirect(array('controller' => 'main', 'function' => 'dashboard'));
	}


	/**
	 * Store the settings if they are correct
	 * or save the error and return to form page
	 *
	 * @token_s string: token generated on revios step
	 */
	public function dropboxAuthorize($token_secret)
	{
		$settings = get_option('pressbackup.preferences');
		$srv = $this->Misc->serviceObject('dropbox', array(null, null));

		//get real tokens
		$token = $_GET['oauth_token'];
		$permanentTokens = $srv->getAccessToken($token, $token_secret);

		//store tokens
		$settings['Service']['dropbox']['credential'] = base64_encode($permanentTokens['oauth_token'] . '|AllYouNeedIsLove|' . $permanentTokens['oauth_token_secret']);
		$settings['Service']['dropbox']['enabled'] = true;

		update_option('pressbackup.preferences', $settings);

		if($settings['configured']){
			$this->Session->write('general_msg', __('Service Saved!','pressbackup'));
			$this->redirect(array('controller' => 'main', 'function' => 'dashboard'));
		} else {
			$this->wizardSetDefaultSettings();
		}
	}

	/**
	 * Check if credentials are valid
	 * for S3 or Pressbackup PRO account
	 *
	 * @args array: credentials sent via post
	 */
	public function checkFieldsService($args)
	{
		if(!isset($args['service'])) {
			$this->Session->write('error_msg', __('Please select at least one storage service','pressbackup'));
			return false;
		}

		$srv = $this->Misc->serviceObject('pressbackup', array(null, null));
		if(count($args['service']) > $srv->getLimit('services')) {
			$this->Session->write('error_msg', __('Please upgrade your membership to allow multiple storage services','pressbackup'));
			return false;
		}

		if (in_array('s3', $args['service'])) {
			$PS3 = $args['s3'];

			//check empty data
			if (!isset($PS3['accessKey']) || empty($PS3['accessKey']) || !isset($PS3['secretKey']) || empty($PS3['secretKey'])) {
				$this->Session->write('error_msg', __('Empty Fields, please ty it again','pressbackup'));
				return false;
			}

			$srv = $this->Misc->serviceObject('s3', array($PS3['accessKey'], $PS3['secretKey']));
			$buckets = $srv->listBuckets();

			//check if credentials are right
			if (!$buckets) {
				$this->Session->write('error_msg', __('Incorrect S3 access or secret Keys','pressbackup'));
				return false;
			}

			//check if bucket exists
			$the_bucket = $this->Config->read('S3.bucketname') . '-' . md5(uniqid(rand(), true));
			$create_bucket = true;

			for ($i = 0; $i < count($buckets); $i++) {
				if (strpos($buckets[$i], $this->Config->read('S3.bucketname')) !== false) {
					$the_bucket = $buckets[$i];
					$create_bucket = false;
					break;
				}
			}

			//create bucket if it not exist
			$region = ($PS3['region'] == 'EU') ? 'EU' : false;
			if ($create_bucket && !$srv->putBucket($the_bucket, ServiceS3PBLib::ACL_PRIVATE, $region)) {
				$this->Session->write('error_msg', __("Unable to create a bucket on S3: Service temporarily unavailable. Please try it again later.",'pressbackup'));
				return false;
			}

			$_POST['data']['s3']['bucketname'] = $the_bucket;
			$_POST['data']['s3']['region'] = $region;
		}


		if (in_array('local', $args['service'])) {

			$PLOCAL = $args['local'];

			$srv = $this->Misc->serviceObject('local', null);


			if (!$srv->checkExists($PLOCAL['path'])) {
				$this->Session->write('error_msg', __("The specified folder does not exist.",'pressbackup'));
				return false;
			}

			if (!$srv->checkPermissions($PLOCAL['path'])) {
				$this->Session->write('error_msg', __("No write permissions for the selected folder",'pressbackup'));
				return false;
			}
		}

		return true;
	}

	/**
	 * Check if fields for Settings are valid.
	 * The files are type of backup, schedule time, etc
	 *
	 * @args array: all fields sent via post
	 */
	public function checkFieldsSettings($args)
	{
		$at_least_one = false;

		//check that all times received they are digit, and that at least one is activated,
		foreach($args['backup'] as $type=>$data){
			if (isset($data['time']) && !preg_match("/^[[:digit:]]+$/", $data['time'])) {
				$this->Session->write('error_msg', __('Specify the scheduled backup time range','pressbackup'));
				return false;
			}
			if($data['time'] != 0){
				$at_least_one = true;
			}
		}

		if(!$at_least_one){
			$this->Session->write('error_msg', __('Specify what you would like to backup','pressbackup'));
			return false;
		}

		return true;
	}

	public function removeUpdateMsg()
	{
		$settings= get_option('pressbackup.preferences');
		$settings['show.updated.msg'] = false;
		update_option('pressbackup.preferences',$settings);
		//reload page
		$this->redirect($_SERVER['HTTP_REFERER']);
	}

}
?>
