<?php


class ServicePressbackupPBLib
{
	private $API_URL = 'https://pressbackup.com/pro';

	private $API_VERSION = '2';

	private $authKey = null;

	/**
	 * Local Copy of SimpleCurl Lib
	 *
	 * @var object
	 * @access public
	 */
	private $Curl = null;

	/**
	 * Local Copy of SimpleCurl response for last call
	 *
	 * @var object
	 * @access public
	 */
	public $response = null;

	/**
	 * Constructor.
	 *
	 * @param object FramePress Core
	 * @access public
	 */
	public function __construct($credentials = null )
	{
		global $pressbackup;

		$pressbackup->import('SimpleCurl.php');
		$this->Curl = new SimpleCurlPBLib();

		$this->setAuth($credentials);
	}

	//----------------------------------------------------------------------------------------

	public function auth()
	{
		return $this->call(array(
			'url' => $this->apiURL('AUTH'),
			'user-agent' => $this->userAgent(),
			'parse.response' => 'json',
			'header' => array(
				$this->gHeader('auth'),
				$this->gHeader('from'),
				$this->gHeader('date'),
			),
		));
	}

	public function putFile($file)
	{
		//fix for no occidental characers, basename and pathinfo remove them from $file
		$pathinfo = explode(DS, $file);
		$name = array_pop($pathinfo);

		return $this->call(array(
			'url' => $this->apiURL('PUTBACKUP'),
			'user-agent' => $this->userAgent(),
			'parse.response' => false,
			'upload.file.PUT' => $file,
			'header' => array(
				$this->gHeader('auth'),
				$this->gHeader('from'),
				$this->gHeader('date'),
				'Content-Type: application/zip',
				'File-Name: '.base64_encode($name)
			),
		));
	}

	public function getFile($file, $saveon)
	{
		return $this->call(array(
			'url' => $this->apiURL('GETBACKUP'),
			'user-agent' => $this->userAgent(),
			'save.output.in' => $saveon,
			'post' => array('file' => $file),
			'header' => array(
				$this->gHeader('auth'),
				$this->gHeader('from'),
				$this->gHeader('date'),
			),
		));
	}

	public function getFile2($file)
	{
		return $this->call(array(
			'url' => $this->apiURL('GETBACKUP2'),
			'user-agent' => $this->userAgent(),
			'parse.response' => 'json',
			'post' => array('file' => $file),
			'header' => array(
				$this->gHeader('auth'),
				$this->gHeader('from'),
				$this->gHeader('date'),
			),
		));
	}

	public  function deleteFile($file)
	{
		return $this->call(array(
			'url' => $this->apiURL('DELETEBACKUP'),
			'user-agent' => $this->userAgent(),
			'parse.response' => false,
			'post' => array('file' => $file),
			'header' => array(
				$this->gHeader('auth'),
				$this->gHeader('from'),
				$this->gHeader('date'),
			),
		));
	}

	public function getFilesList()
	{
		return $this->call(array(
			'url' => $this->apiURL('GETBACKUPSLIST'),
			'user-agent' => $this->userAgent(),
			'parse.response' => 'json',
			'header' => array(
				$this->gHeader('auth'),
				$this->gHeader('from'),
				$this->gHeader('date'),
			),
		));
	}

	public function getLimit($limit = null)
	{
		$res = $this->call(array(
			'url' => $this->apiURL('GETLIMIT'),
			'user-agent' => $this->userAgent(),
			'parse.response' => 'json',
			'post' => array('limit_type' => $limit),
			'header' => array(
				$this->gHeader('auth'),
				$this->gHeader('from'),
				$this->gHeader('date'),
			),
		));
		return $res[$limit];
	}

	public function createAccount($type = null)
	{
		$ud=get_userdata(get_current_user_id());

		$info = array( 'User' => array(
			'username' => $_POST['data']['User']['username'],
			'password' => md5($_POST['data']['User']['password']),
			'firstname' => $_POST['data']['User']['first_name'],
			'lastname' => $_POST['data']['User']['last_name'],
			'email' => $ud->data->user_email,
		));

		return $this->call(array(
			'url' => $this->apiURL('CREATEUSER'),
			'user-agent' => $this->userAgent(),
			'parse.response' => 'json',
			'post' => array('user' => base64_encode(json_encode($info))),
			'header' => array(
				$this->gHeader('auth'),
				$this->gHeader('from'),
				$this->gHeader('date'),
			),
		));
	}

	public function registerSite ()
	{
		global $pressbackup;
		$pressbackup->redirect($this->API_URL . DS . 'sites' . DS . 'registerSite');
	}

	public function pingInstalled()
	{
		return $this->call(array(
			'url' => $this->apiURL('PING.INSTALL'),
			'user-agent' => $this->userAgent(),
			'parse.response' => false,
			'header' => array(
				$this->gHeader('auth'),
				$this->gHeader('from'),
				$this->gHeader('date'),
			),
		));
	}

	public function pingConfigured()
	{
		return $this->call(array(
			'url' => $this->apiURL('PING.CONFIG'),
			'user-agent' => $this->userAgent(),
			'parse.response' => false,
			'header' => array(
				$this->gHeader('auth'),
				$this->gHeader('from'),
				$this->gHeader('date'),
			),
		));
	}

	public function ping()
	{
		return $this->call(array(
			'url' => $this->apiURL('PING'),
			'user-agent' => $this->userAgent(),
			'parse.response' => false,
			'header' => array(
				$this->gHeader('auth'),
				$this->gHeader('from'),
				$this->gHeader('date'),
			),
		));
	}

	//----------------------------------------------------------------------------------------

	public function call ($args = array())
	{
		$res = $this->Curl->call($args);
		$this->response = $this->Curl->response;
		return $res;
	}

	public function setAuth( $credentials = null)
	{
		if (!$credentials[0] || !$credentials[1] ){ return true; }
		$this->authKey=base64_encode($credentials[0].','.$credentials[1]);
	}

	private function apiURL( $action = null ){
		return $this->API_URL . '/api/' . $this->API_VERSION . '/' . $action;
	}

	private function gHeader( $type = null )
	{
		if ($type == 'from'){ return 'From: '.get_bloginfo( 'wpurl' ); }
		if ($type == 'auth' && $this->authKey){ return 'Auth: '.$this->authKey; }
		if ($type == 'date'){ return 'Date: '.date(DATE_ATOM); }
		return 'Forgot: toPassType';
	}

	private function userAgent()
	{
		return 'Presbackup/plugin/service/PHP '.get_bloginfo( 'wpurl' );
	}
}
